@@ -1,5 +1,63 @@
-from aestate.exception import e_fields
+from typing import List
+
+from aestate.exception import e_fields, SqlResultError
 from aestate.util.Log import CACodeLog
+from prettytable import PrettyTable
+
+from aestate.work.Serialize import QuerySet
+
+
+class BaseCover:
+
+    def table_visual(self, title: list, val: List[dict]) -> PrettyTable:
+        if not val:
+            raise SqlResultError("数据库获取表内字段类型的sql无返回,sql编写错误,请检查sql")
+
+        table = PrettyTable(title)
+        table.border = True
+        [table.add_row(list(i.values())) for i in val]
+        return table
+
+    def check(self, res, fields) -> bool:
+        """
+        检查表与数据库中的数据是否对应
+        """
+        comment = {
+            'auto_increment': "自增",
+            'DEFAULT_GENERATED': "自动追加日期",
+            'DEFAULT_GENERATED on update CURRENT_TIMESTAMP': "自动更新时间为最后一次更改时间"
+        }
+        return False
+
+    def deal(self, res, fields):
+        """
+        同步数据库与pojo的字段配置项
+        """
+        return True
+
+    def res(self, tb_name, db_name, extool) -> dict:
+        """
+        获得表结构
+        """
+        sql = """SELECT 
+        COLUMN_NAME AS name, -- 名称
+        DATA_TYPE AS typer, -- 类型
+        CHARACTER_MAXIMUM_LENGTH AS length, -- 长度
+        NUMERIC_SCALE AS num_scale, -- 数字小数点
+        IS_NULLABLE AS is_null, -- 是否允许为空
+        COLUMN_KEY AS c_key, -- 是否为键
+        EXTRA as def_val, -- 默认值
+        COLUMN_COMMENT as comment -- 描述
+        FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s"""
+        # 使用传入的sql执行器执行查询表结构的sql
+        res = extool.select(sql=sql, params=[db_name, tb_name])
+        # 获得表结构可视化
+        # table = self.table_visual(list(res[0].keys()), res)
+        # print(table)
+
+        # 对比结构
+
+        return res
 
 
 class CACodePureORM(object):
@@ -34,14 +92,17 @@ s        """
             CACodeLog.err(
                 AttributeError, 'Repository is null,Place use repository of ORM framework')
         self.repository = repository
-        self.__table_name__ = '{}{}{}'.format(self.sqlFields.left_subscript, repository.__table_name__,
-                                              self.sqlFields.right_subscript)
+        # self.__table_name__ = '{}{}{}'.format(self.sqlFields.left_subscript, repository.__table_name__,
+        #                                       self.sqlFields.right_subscript)
+        self.__table_name__ = repository.__table_name__
 
         self.first_data = False
         self._result = []
 
     def top(self):
-        return self.find().limit(1)
+        self.find()
+        self.first_data = True
+        return self.limit(1)
 
     def first(self):
         """
@@ -51,45 +112,6 @@ s        """
         return self
 
     # ------------------------主键--------------------------
-
-    def insert(self, pojo):
-        """
-        插入
-        example:
-            insert()
-            insert('c1','c2')
-        :param pojo:需要插入的对象
-        """
-        # 添加insert关键字
-        # self.args.append(insert_str)
-        # self.args.append('{}{}'.format(self.__table_name__, left_par))
-        sql = self.ParseUtil.parse_insert_pojo(
-            pojo, self.__table_name__.replace('`', ''), insert_str=self.sqlFields.insert_str,
-            values_str=self.sqlFields.values_str)
-        self.args.append(sql['sql'])
-        self.params = sql['params']
-        return self
-
-    def delete(self):
-        """
-        删除
-        """
-        self.args.append(self.sqlFields.delete_str)
-        self.args.append(self.sqlFields.from_str)
-        self.args.append(self.__table_name__)
-        return self
-
-    def update(self):
-        """
-        更新
-        example:
-            update().set(key=value).where(key1=value1)
-        """
-        update_sql = '%s%s' % (self.sqlFields.update_str,
-                               str(self.__table_name__))
-        self.args.append(update_sql)
-        return self
-
     def find(self, *args, **kwargs):
         """
         查
@@ -104,9 +126,9 @@ s        """
         """
         self.args.append(self.sqlFields.find_str)
         # 如果有as字段
-        asses = None
-        if 'asses' in kwargs.keys():
-            asses = kwargs['asses']
+        alias = None
+        if 'alias' in kwargs.keys():
+            alias = kwargs['alias']
         # 如果包含方法的字段，则不加密
         func_flag = False
         if 'h_func' in kwargs.keys():
@@ -129,26 +151,26 @@ s        """
             if not func_flag:
                 fields = self.ParseUtil.parse_key(*args, is_field=True)
             else:
-                fields = self.ParseUtil.parse_key(*args)
+                fields = self.ParseUtil.parse_key(*args, is_field=False)
         # 解决as问题
-        if asses is not None:
+        if alias is not None:
             fs = fields.split(',')
-            if len(fs) != len(asses):
+            if len(fs) != len(alias):
                 # 匿名参数长度与字段长度不符合
                 CACodeLog.log_error(obj=TypeError,
                                     msg='The length of the anonymous parameter does not match the length of the field',
                                     raise_exception=True)
             for i, v in enumerate(fs):
-                if asses[i] is not None:
+                if alias[i] is not None:
                     self.args.append('{}{}{}'.format(
-                        v, self.sqlFields.asses_str, asses[i]))
+                        v, self.sqlFields.asses_str, alias[i]))
                 else:
                     self.args.append(v)
                 # 逗号
                 self.args.append(self.sqlFields.comma)
         else:
             self.args.append(fields)
-        if asses is not None:
+        if alias is not None:
             # 去掉末尾的逗号
             self.rep_sym()
         #     加上from关键字
@@ -158,7 +180,7 @@ s        """
             self.args += kwargs['poly']
         return self
 
-    def order_by(self, *args):
+    def order_by(self, *args, **kwargs):
         """
         根据什么查
         example:
@@ -166,29 +188,34 @@ s        """
             find('all').order_by('param').end()
             find('all').order_by('p1','p2').desc().limit(10,20)
         """
-        return self.by_opera(field=self.sqlFields.order_by_str, args_list=args)
+        return self.by_opera(field=self.sqlFields.order_by_str, args_list=args, **kwargs)
 
-    def group_by(self, *args):
+    def group_by(self, *args, **kwargs):
         """
         聚合函数
         example:
             select shop_id,count(*) as count from comments group by shop_id having count>1;
         """
-        return self.by_opera(field=self.sqlFields.group_by_str, args_list=args)
+        return self.by_opera(field=self.sqlFields.group_by_str, args_list=args, **kwargs)
 
-    def by_opera(self, field, args_list):
+    def by_opera(self, field, args_list, **kwargs):
         """
         根据什么查
         """
         self.args.append(field)
         for i in args_list:
-            self.args.append(self.sqlFields.left_subscript)
+            if not kwargs.get('text', False):
+                self.args.append(self.sqlFields.left_subscript)
             self.args.append(i)
-            self.args.append(self.sqlFields.right_subscript)
-            self.args.append(self.sqlFields.ander_str)
-        self.rep_sym(self.sqlFields.ander_str, self.sqlFields.space)
+            if not kwargs.get('text', False):
+                self.args.append(self.sqlFields.right_subscript)
+            self.args.append(self.sqlFields.comma)
+        self.rep_sym(self.sqlFields.comma, self.sqlFields.space)
         return self
 
+    def filter(self, **kwargs):
+        return self.find().where(**kwargs).end()
+
     def where(self, **kwargs):
         """
         当....
@@ -243,29 +270,44 @@ s        """
                 self.ParseUtil.adapter.funcs[sym](self, cp_key, value)
 
             if not customize:
-                self.args.append('`{}`{}%s'.format(cp_key, sym))
+                self.args.append(
+                    '{}{}{}{}%s'.format(self.sqlFields.left_subscript,
+                                        cp_key,
+                                        self.sqlFields.right_subscript,
+                                        sym))
                 self.args.append(self.sqlFields.ander_str)
                 self.params.append(value)
         self.rep_sym(self.sqlFields.ander_str)
 
         return self
 
-    def limit(self, star=0, end=None):
+    def on(self, from_where, to_where, symbol='='):
+        self.args.append(self.sqlFields.space)
+        self.args.append(self.sqlFields.on_str)
+        self.args.append(self.sqlFields.space)
+        self.args.append(from_where)
+        self.args.append(self.sqlFields.space)
+        self.args.append(symbol)
+        self.args.append(self.sqlFields.space)
+        self.args.append(to_where)
+        return self
+
+    def limit(self, start=0, end=None):
         """
         分页
-        :param star:开始
+        :param start:开始
         :param end:末尾
         example:
-            find('all').limit(star=10,end=20)
+            find('all').limit(start=10,end=20)
             find('all').limit(end=10)
         """
         self.args.append(self.sqlFields.limit_str)
         # 死亡空格
         if end is None:
             limit_param = '{}{}{}'.format(
-                self.sqlFields.space, star, self.sqlFields.space)
+                self.sqlFields.space, start, self.sqlFields.space)
         else:
-            limit_param = '{}{}{}{}{}'.format(self.sqlFields.space, star, self.sqlFields.comma, end,
+            limit_param = '{}{}{}{}{}'.format(self.sqlFields.space, start, self.sqlFields.comma, end,
                                               self.sqlFields.space)
         self.args.append(limit_param)
         return self
@@ -297,7 +339,10 @@ s        """
         self.args.append(self.sqlFields.set_str)
         _size = len(kwargs.keys())
         for key, value in kwargs.items():
-            self.args.append('`{}`{}%s'.format(key, self.sqlFields.eq))
+            self.args.append('{}{}{}{}%s'.format(self.sqlFields.left_subscript,
+                                                 key,
+                                                 self.sqlFields.right_subscript,
+                                                 self.sqlFields.eq))
             # set是加逗号不是and
             self.args.append(self.sqlFields.comma)
             self.params.append(value)
@@ -317,7 +362,7 @@ s        """
         self.args.append(self.sqlFields.ander_str)
         return self
 
-    def run(self, need_sql=False, serializer=True, **kwargs):
+    def run(self, need_sql=False, serializer=True) -> QuerySet:
         """
         最终执行任务
         """
@@ -356,15 +401,18 @@ s        """
                     self._result) > 0:
                 if not serializer:
                     return self._result[0]
-                return self.serializer(instance=self.repository.instance, base_data=self._result).first()
+                self.repository.result = self.serializer(instance=self.repository.instance,
+                                                         base_data=self._result).first()
+                return self.repository.result
             else:
-                return None
+                self.repository.result = self.serializer()
+                return self.repository.result
         else:
             if not serializer:
                 return self._result
-            q = self.serializer(
+            self.repository.result = self.serializer(
                 instance=self.repository.instance, base_data=self._result)
-            return q
+            return self.repository.result
 
     def con_from(self):
         """
@@ -373,7 +421,9 @@ s        """
         if self.sqlFields.from_str not in self.args:
             self.args.append(self.sqlFields.from_str)
             # 然后加上表名
+            self.args.append(self.sqlFields.left_subscript)
             self.args.append(self.__table_name__)
+            self.args.append(self.sqlFields.right_subscript)
 
     def append(self, app_sql):
         """
@@ -413,11 +463,31 @@ s        """
         self.args = self.args + new_args
         return self
 
-    def serializer(self):
-        q = self.serializer(
-            instance=self.repository.instance, base_data=self._result)
-        return q
+    def alias(self, name):
+        """
+        设置别名
+        """
+        self.args.append(' AS ')
+        self.args.append(name)
+        return self
+
+    def left_join(self, sql_orm, name):
+        """
+        left join
+        """
+        self.args.append(self.sqlFields.left_join_str)
+        self.args.append(sql_orm.__table_name__)
+        return self.alias(name)
 
     def __str__(self):
         sql = ''.join(self.args)
         return sql
+
+    def check(self):
+        """sudo apt install containerd
+        检查表结构与数据库中是否对应
+        """
+        return self.repository.config_obj.opera(self.repository).check()
+
+    def create(self):
+        return self.repository.config_obj.opera(self.repository).create()
